<?php
/**
 * alpenhouse Theme Customizer
 *
 * @package alpenhouse
 */

/**
 * Add postMessage support for site title and description for the Theme Customizer.
 *
 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
 */
function alpenhouse_customize_register( $wp_customize ) {
	$wp_customize->get_setting( 'blogname' )->transport         = 'postMessage';
	$wp_customize->get_setting( 'blogdescription' )->transport  = 'postMessage';
	$wp_customize->get_setting( 'header_textcolor' )->transport = 'postMessage';

	if ( isset( $wp_customize->selective_refresh ) ) {
		$wp_customize->selective_refresh->add_partial( 'blogname', array(
			'selector'        => '.site-title a',
			'render_callback' => 'alpenhouse_customize_partial_blogname',
		) );
		$wp_customize->selective_refresh->add_partial( 'blogdescription', array(
			'selector'        => '.site-description',
			'render_callback' => 'alpenhouse_customize_partial_blogdescription',
		) );
	}
}
add_action( 'customize_register', 'alpenhouse_customize_register' );

/**
 * Render the site title for the selective refresh partial.
 *
 * @return void
 */
function alpenhouse_customize_partial_blogname() {
	bloginfo( 'name' );
}

/**
 * Render the site tagline for the selective refresh partial.
 *
 * @return void
 */
function alpenhouse_customize_partial_blogdescription() {
	bloginfo( 'description' );
}

/**
 * Binds JS handlers to make Theme Customizer preview reload changes asynchronously.
 */
function alpenhouse_customize_preview_js() {
	wp_enqueue_script( 'alpenhouse-customizer', get_template_directory_uri() . '/js/customizer.js', array( 'customize-preview' ), alpenhouse_get_theme_version(), true );
}
add_action( 'customize_preview_init', 'alpenhouse_customize_preview_js' );

/**
 * Adds custom panels to customizer.
 */

function alpenhouse_customizer_add_settings($wp_customize)
{

    $wp_customize->add_panel( 'theme_options', array(
        'title' => __( 'Theme Options' ),
        'priority' => 30,
    ) );

    /**
     * Header settings
     */
    $wp_customize->add_section('alpenhouse_header', array(
        'title' => __('Header Options', 'alpenhouse'),
        'panel' => 'theme_options'
    ));
    $wp_customize->add_setting('alpenhouse_display_header_search', array(
        'default' => '1',
        'sanitize_callback' => 'alpenhouse_sanitize_checkbox'
    ));
    $wp_customize->add_control('alpenhouse_display_header_search', array(
        'label' => __('Display site search form in header.', 'alpenhouse'),
        'section' => 'alpenhouse_header',
        'type' => 'checkbox'
    ));

    $wp_customize->add_setting('alpenhouse_subpages_header_overlay', array(
        'default' => ALPENHOUSE_SUBPAGES_HEADER_OVERLAY,
        'transport' => 'postMessage',
        'sanitize_callback' => 'sanitize_hex_color',
    ));
    $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'alpenhouse_subpages_header_overlay', array(
        'label' => esc_html__('Subpage Header Overlay Color', 'alpenhouse'),
        'section' => 'alpenhouse_header',
        'priority' => 19,
    )));

    $wp_customize->add_setting('alpenhouse_subpages_header_overlay_opacity', array(
        'default' => ALPENHOUSE_SUBPAGES_HEADER_OVERLAY_OPACITY,
        'transport' => 'postMessage',
        'sanitize_callback' => 'absint',
    ));
    $wp_customize->add_control( 'alpenhouse_subpages_header_overlay_opacity', array(
        'label' => esc_html__('Subpage Header Overlay Opacity', 'alpenhouse'),
        'type' => 'range',
        'section' => 'alpenhouse_header',
        'priority' => 20,
        'input_attrs' => array(
            'min' => 0,
            'max' => 100,
            'step' => 1,
        ),
    ));

	$wp_customize->add_setting('alpenhouse_sticky_header', array(
		'default' => false,
		'sanitize_callback' => 'alpenhouse_sanitize_checkbox'
	));
	$wp_customize->add_control('alpenhouse_sticky_header', array(
		'label' => __('Make header sticky?', 'alpenhouse'),
		'section' => 'alpenhouse_header',
		'type' => 'checkbox',
		'priority' => 99 // move control to end
	));

	$wp_customize->add_setting('alpenhouse_sticky_header_on_scroll', array(
		'default' => '1',
		'sanitize_callback' => 'alpenhouse_sanitize_checkbox'
	));
	$wp_customize->add_control('alpenhouse_sticky_header_on_scroll', array(
		'label' => __('Enable slide-out/slide-in header behaviour on scroll?', 'alpenhouse'),
		'section' => 'alpenhouse_header',
		'type' => 'checkbox',
		'description' => __('Works only with enabled sticky header.', 'alpenhouse'),
		'priority' => 100 // move control to end
	));


	/**
     * Footer settings
     */
    $wp_customize->add_section('alpenhouse_footer', array(
        'title' => __('Footer Options', 'alpenhouse'),
        'panel' => 'theme_options'
    ));
    $wp_customize->add_setting('alpenhouse_footer_text', array(
        'default' => esc_html_x('%2$s &copy; %1$s ', 'Default footer text. %1$s - current year, %2$s - site title.', 'alpenhouse'),
        'transport' => 'postMessage',
        'type' => 'theme_mod',
        'sanitize_callback' => 'wp_kses_post'
    ));
    $wp_customize->add_control('alpenhouse_footer_text', array(
        'label' => esc_html__('Footer Text', 'alpenhouse'),
        'description' => esc_html__('Use %1$s to insert the current year and %2$s - the site title. Doesn`t work for Live Preview.', 'alpenhouse'),
        'section' => 'alpenhouse_footer',
        'type' => 'textarea',
        'settings' => 'alpenhouse_footer_text'
    ));


    if ( class_exists( 'WooCommerce' ) ) {
        $wp_customize->add_setting('alpenhouse_display_header_cart', array(
            'default' => '1',
            'sanitize_callback' => 'alpenhouse_sanitize_checkbox'
        ));
        $wp_customize->add_control('alpenhouse_display_header_cart', array(
            'label' => __('Display WooCommerce cart in header.', 'alpenhouse'),
            'section' => 'alpenhouse_header',
            'type' => 'checkbox'
        ));
    }


    /**
     * Blog settings
     */
    $wp_customize->add_section( 'alpenhouse_blog' , array(
        'title'      => __( 'Blog Options', 'alpenhouse' ),
        'panel' => 'theme_options'
    ) );
    $wp_customize->add_setting('alpenhouse_blog_type', array(
        'default' => ALPENHOUSE_BLOG_LAYOUT_1,
        'transport' => 'refresh',
        'type' => 'theme_mod',
        'sanitize_callback' => 'alpenhouse_sanitize_select'

    ));
    $wp_customize->add_control('alpenhouse_blog_type', array(
        'label' => __('Blog Layout', 'alpenhouse' ),
        'type'      => 'select',
        'section'   => 'alpenhouse_blog',
        'choices'   => array(
            ALPENHOUSE_BLOG_LAYOUT_1 => __('Classic with sidebar', 'alpenhouse'),
            ALPENHOUSE_BLOG_LAYOUT_2 => __('Classic no sidebar', 'alpenhouse'),
            ALPENHOUSE_BLOG_LAYOUT_3 => __('Masonry 2 columns', 'alpenhouse'),
            ALPENHOUSE_BLOG_LAYOUT_4 => __('Masonry 3 columns', 'alpenhouse'),
        ),
        'settings'  => 'alpenhouse_blog_type'

    ));

    $wp_customize->add_setting('alpenhouse_post_overlay', array(
        'default' => ALPENHOUSE_THUMBNAIL_OVERLAY,
        'transport' => 'postMessage',
        'sanitize_callback' => 'sanitize_hex_color',
    ));
    $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'alpenhouse_post_overlay', array(
        'label' => esc_html__('Post Thumbnail Hover Overlay Color', 'alpenhouse'),
        'section' => 'alpenhouse_blog',
        'priority' => 19,
    )));

    $wp_customize->add_setting('alpenhouse_post_overlay_opacity', array(
        'default' => ALPENHOUSE_THUMBNAIL_OVERLAY_OPACITY,
        'transport' => 'postMessage',
        'sanitize_callback' => 'absint',
    ));
    $wp_customize->add_control( 'alpenhouse_post_overlay_opacity', array(
        'label' => esc_html__('Post Thumbnail Hover Overlay Opacity', 'alpenhouse'),
        'type' => 'range',
        'section' => 'alpenhouse_blog',
        'priority' => 20,
        'input_attrs' => array(
            'min' => 0,
            'max' => 100,
            'step' => 1,
        ),
    ));


    /**
     * Portfolio settings
     */
    $wp_customize->add_section( 'alpenhouse_portfolio' , array(
        'title'      => __( 'Gallery Options', 'alpenhouse' ),
        'panel' => 'theme_options'
    ) );
    $wp_customize->add_setting('alpenhouse_portfolio_columns', array(
        'default' => 2,
        'transport' => 'refresh',
        'type' => 'theme_mod',
        'sanitize_callback' => 'alpenhouse_sanitize_select'

    ));
    $wp_customize->add_control('alpenhouse_portfolio_columns', array(
        'label' => __('Gallery Layout', 'alpenhouse' ),
        'type'      => 'select',
        'section'   => 'alpenhouse_portfolio',
        'choices'   => array(
            2 => __('Default', 'alpenhouse'),
            3 => __('3 columns', 'alpenhouse'),
            4 => __('4 columns', 'alpenhouse'),
        ),
        'settings'  => 'alpenhouse_portfolio_columns'

    ));

    /**
     * Booking settings
     */
    if(class_exists('HotelBookingPlugin')) {

        $wp_customize->add_section('alpenhouse_booking', array(
            'title' => __('Booking Options', 'alpenhouse'),
            'panel' => 'theme_options'
        ));
        $wp_customize->add_setting('alpenhouse_accommodation_list_layout', array(
            'default' => ALPENHOUSE_ACCOMMODATION_LIST_LAYOUT_1,
            'transport' => 'refresh',
            'type' => 'theme_mod',
            'sanitize_callback' => 'alpenhouse_sanitize_select'

        ));
        $wp_customize->add_control('alpenhouse_accommodation_list_layout', array(
            'label' => __('Accommodations Layout', 'alpenhouse'),
            'type' => 'select',
            'section' => 'alpenhouse_booking',
            'choices' => array(
                ALPENHOUSE_ACCOMMODATION_LIST_LAYOUT_1 => __('Default', 'alpenhouse'),
                ALPENHOUSE_ACCOMMODATION_LIST_LAYOUT_2 => __('Grid', 'alpenhouse'),
                ALPENHOUSE_ACCOMMODATION_LIST_LAYOUT_3 => __('List', 'alpenhouse'),
            ),
            'settings' => 'alpenhouse_accommodation_list_layout'

        ));
    }

    /**
     *  404 Page settings
     */
    $wp_customize->add_section( 'alpenhouse_not_found' , array(
        'title'      => __( '404 Page', 'alpenhouse' ),
        'panel' => 'theme_options'
    ) );
    $wp_customize->add_setting('alpenhouse_not_found_image', array(
        'default' => get_template_directory_uri().'/img/404_header.jpg',
        'type' => 'theme_mod',
        'transport' => 'refresh',
        'sanitize_callback' => 'alpenhouse_sanitize_image'
    ));
    $wp_customize->add_control(new WP_Customize_Image_Control(
            $wp_customize,
            'alpenhouse_not_found_image',
            array(
                'label'      => __( 'Background Image', 'alpenhouse' ),
                'section'    => 'alpenhouse_not_found',
                'settings'   => 'alpenhouse_not_found_image',
            )
        )
    );
    /**
     * Check if WooCommerce is active
     **/
    if ( class_exists( 'WooCommerce' ) ) {
        $wp_customize->add_panel('alpenhouse_shop_panel',array(
            'title'      => __('WooCommerce Options', 'alpenhouse'),
            'panel' => 'theme_options'
        ));
        $wp_customize->add_section('alpenhouse_shop', array(
            'title'     => __('Shop Layout', 'alpenhouse'),
            'priority'  => 1,
            'panel'     => 'alpenhouse_shop_panel',
        ));
        $wp_customize->add_setting('alpenhouse_shop_type', array(
            'default'   => ALPENHOUSE_SHOP_LAYOUT_1,
            'transport' => 'refresh',
            'type'      => 'theme_mod',
            'sanitize_callback' => 'alpenhouse_sanitize_select'
        ));
        $wp_customize->add_control('alpenhouse_shop_type', array(
            'label'     => __('Shop Layout', 'alpenhouse'),
            'type'      => 'select',
            'section'   => 'alpenhouse_shop',
            'choices'   => array(
                ALPENHOUSE_SHOP_LAYOUT_1  => __('Classic with sidebar', 'alpenhouse'),
                ALPENHOUSE_SHOP_LAYOUT_2  => __('Without sidebar', 'alpenhouse'),
            ),
            'settings'  => 'alpenhouse_shop_type'
        ));
        $wp_customize->add_setting('alpenhouse_shop_products_per_page', array(
            'default'   => 9,
            'transport' => 'refresh',
            'type'      => 'theme_mod',
            'sanitize_callback' => 'absint'
        ));
        $wp_customize->add_control('alpenhouse_shop_products_per_page', array(
            'label'     => __('Products per page', 'alpenhouse'),
            'type'      => 'number',
            'section'   => 'alpenhouse_shop',
            'settings'  => 'alpenhouse_shop_products_per_page'
        ));



        $wp_customize->add_section('alpenhouse_single_product', array(
            'title'     => __('Single Product Layout', 'alpenhouse'),
            'priority'  => 2,
            'panel'     => 'alpenhouse_shop_panel',
        ));
        $wp_customize->add_setting('alpenhouse_single_product_layout', array(
            'default'   => ALPENHOUSE_SINGLE_PRODUCT_LAYOUT_1,
            'transport' => 'refresh',
            'type'      => 'theme_mod',
            'sanitize_callback' => 'alpenhouse_sanitize_select'
        ));
        $wp_customize->add_control('alpenhouse_single_product_layout', array(
            'label'     => __('Single Product Layout', 'alpenhouse'),
            'type'      => 'select',
            'section'   => 'alpenhouse_single_product',
            'choices'   => array(
                ALPENHOUSE_SINGLE_PRODUCT_LAYOUT_1 => __('Classic with sidebar', 'alpenhouse'),
                ALPENHOUSE_SINGLE_PRODUCT_LAYOUT_2  => __('Without sidebar', 'alpenhouse'),
            ),
            'settings'  => 'alpenhouse_single_product_layout'

        ));
    }

    /**
     * COLOR SETTINGS
     */

    $wp_customize->add_setting('alpenhouse_accent_color', array(
        'default' => ALPENHOUSE_ACCENT_COLOR,
        'transport' => 'postMessage',
        'sanitize_callback' => 'sanitize_hex_color',
    ));
    $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'alpenhouse_accent_color', array(
        'label' => esc_html__('Accent Color', 'alpenhouse'),
        'section' => 'colors',
    )));

    $wp_customize->add_setting('alpenhouse_secondary_color', array(
        'default' => ALPENHOUSE_SECONDARY_COLOR,
        'transport' => 'postMessage',
        'sanitize_callback' => 'sanitize_hex_color',
    ));
    $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'alpenhouse_secondary_color', array(
        'label' => esc_html__('Secondary Color', 'alpenhouse'),
        'section' => 'colors',
    )));

    $wp_customize->add_setting('alpenhouse_third_color', array(
        'default' => ALPENHOUSE_THIRD_COLOR,
        'transport' => 'postMessage',
        'sanitize_callback' => 'sanitize_hex_color',
    ));
    $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'alpenhouse_third_color', array(
        'label' => esc_html__('Third Color', 'alpenhouse'),
        'section' => 'colors',
    )));


}
add_action( 'customize_register', 'alpenhouse_customizer_add_settings' );

function alpenhouse_sanitize_checkbox( $input ){
    if ($input == 1) {
        return 1;
    } else {
        return '';
    }
}

function alpenhouse_sanitize_select( $input, $setting ){

    //input must be a slug: lowercase alphanumeric characters, dashes and underscores are allowed only
    $input = sanitize_key($input);

    //get the list of possible select options
    $choices = $setting->manager->get_control( $setting->id )->choices;

    //return input if valid or return default option
    return ( array_key_exists( $input, $choices ) ? $input : $setting->default );

}

function alpenhouse_sanitize_image( $image, $setting ) {
    /*
     * Array of valid image file types.
     *
     * The array includes image mime types that are included in wp_get_mime_types()
     */
    $mimes = array(
        'jpg|jpeg|jpe' => 'image/jpeg',
        'gif'          => 'image/gif',
        'png'          => 'image/png',
        'bmp'          => 'image/bmp',
        'tif|tiff'     => 'image/tiff',
    );
    // Return an array with file extension and mime_type.
    $file = wp_check_filetype( $image, $mimes );
    // If $image has a valid mime_type, return it; otherwise, return the default.
    return ( $file['ext'] ? $image : $setting->default );
}

function alpenhouse_generate_customizer_css(){

    $css = '';

    $color = get_theme_mod('alpenhouse_accent_color', ALPENHOUSE_ACCENT_COLOR);
    if( $color !== ALPENHOUSE_ACCENT_COLOR ){

        $css .= '
            input[type="button"],
            input[type="reset"],
            input[type="submit"],
            button[type="submit"],
            .button,
            .menu-highlighted-item a,
            .more-link,
            input[type="button"].secondary:hover,
            input[type="reset"].secondary:hover,
            input[type="submit"].secondary:hover,
            button[type="submit"].secondary:hover,
            .button.secondary:hover,
            .menu-highlighted-item a.secondary:hover,
            .more-link.secondary:hover,
            .pagination a.page-numbers:hover,
            .pagination a.prev:hover,
            .pagination a.next:hover,
            .testimonials-carousel .slick-dots li:hover button,
            .testimonials-carousel .slick-dots li.slick-active button,
            .elementor-widget-image-carousel .elementor-slick-slider .slick-arrow:hover,
            #tribe-bar-form .tribe-bar-submit input[type="submit"],
            .site .site-wrapper .site-header .site-header-wrapper .main-slider-content .slick-prev:hover, 
            .site .site-wrapper .site-header .site-header-wrapper .main-slider-content .slick-next:hover,            
            .portfolio-menu ul li.current-cat a,
            .mphb-room-types .mphb-room-type .mphb-view-details-button-wrapper a:hover, 
            .mphb_sc_search_results-wrapper .mphb-room-type .mphb-view-details-button-wrapper a:hover,
            .mphb-flexslider ul.flex-direction-nav .flex-prev:hover, 
            .mphb-flexslider ul.flex-direction-nav .flex-next:hover,
            .mphb-calendar .datepick-nav .datepick-cmd:hover, 
            .datepick-popup .datepick-nav .datepick-cmd:hover,
            .datepick-popup .datepick-month table tbody tr td .datepick-highlight, 
            .datepick-popup .datepick-month table tbody tr td .mphb-check-in-date                       
            {
                background: '.$color.';
            }
            a, a:hover, a:focus, a:active,
            .footer-navigation ul li a,
            .footer-navigation ul li a:hover,
            .footer-navigation ul li a:visited,
            .searchform .search-controls button:hover,
            .widget_calendar .calendar_wrap table tr td a,
            .widget_calendar .calendar_wrap table tr td a:hover,
            .widget_calendar .calendar_wrap table tfoot td,
            .widget_calendar .calendar_wrap table caption,
            .widget .alpenhouse-social-menu ul li a:hover:before,
            .blog-posts-widget-wrapper .post .entry-header h2 a:hover,
            .pricing-item .pricing-subtitle,
            .elementor-widget-image-carousel .elementor-slick-slider .slick-dots li button:hover:before,
            .elementor-widget-image-carousel .elementor-slick-slider .slick-dots li.slick-active button:before,
            #tribe-events-content .tribe-events-tooltip h4, #tribe_events_filters_wrapper
            .tribe_events_slider_val, .single-tribe_events a.tribe-events-gcal,
            .single-tribe_events a.tribe-events-ical,
            .site .site-wrapper .site-header .page-header-custom .breadcrumbs ul li a:hover,
            .hentry:not(.page) .entry-header h2 a:hover,
            .hentry:not(.page) .entry-header .featured-post,
            .entry-meta a,
            .entry-header .entry-title a:hover,
            .related-posts .related-post a:hover .title,
            .related-posts .related-post .entry-title:hover,
            body.archive .entry-header .entry-title a:hover,
            #main-slider-dots .slick-dots li button:hover:before, #main-slider-fixed-dots .slick-dots li button:hover:before,
            #main-slider-dots .slick-dots li.slick-active button:before, #main-slider-fixed-dots .slick-dots li.slick-active button:before,
            .testimonials-wrapper .testimonial-wrapper .rating .star, 
            .testimonials-carousel .testimonial-wrapper .rating .star, 
            .single-cptp-testimonial .cptp-testimonial .testimonial-wrapper .rating .star,
            .theme-social-menu a:hover,
            .mphb-room-types .mphb-room-type .mphb-room-type-title a:hover, 
            .mphb_sc_search_results-wrapper .mphb-room-type .mphb-room-type-title a:hover,
            .mphb-calendar .datepick-ctrl a, 
            .datepick-popup .datepick-ctrl a,
            .mphb_sc_services-wrapper .mphb-service-title a:hover,
            .widget.widget_mphb_rooms_widget .mphb-widget-room-type-title a:hover,
            .main-navigation .current_page_item > a, 
            .main-navigation .current-menu-item > a, 
            .main-navigation .current_page_ancestor > a,
            .main-navigation .current-menu-ancestor > a, 
            .footer-navigation .current_page_item > a, 
            .footer-navigation .current-menu-item > a, 
            .footer-navigation .current_page_ancestor > a, 
            .footer-navigation .current-menu-ancestor > a,
            .main-navigation li:hover > a, 
            .main-navigation li.focus > a, 
            .footer-navigation li:hover > a, 
            .footer-navigation li.focus > a,
            .mphb_sc_account .mphb-account-menu a:hover 
            {
                color: '.$color.';
            }
            @media screen and (max-width: 991px) {
              .main-navigation ul ul :hover > a,
              .main-navigation ul ul .focus > a,
              .footer-navigation ul ul :hover > a,
              .footer-navigation ul ul .focus > a,
              .main-navigation ul ul a:hover,
              .main-navigation ul ul a.focus,
              .footer-navigation ul ul a:hover,
              .footer-navigation ul ul a.focus,
              .main-navigation ul ul .current_page_item > a,
              .main-navigation ul ul .current-menu-item > a,
              .main-navigation ul ul .current_page_ancestor > a,
              .main-navigation ul ul .current-menu-ancestor > a,
              .footer-navigation ul ul .current_page_item > a,
              .footer-navigation ul ul .current-menu-item > a,
              .footer-navigation ul ul .current_page_ancestor > a,
              .footer-navigation ul ul .current-menu-ancestor > a,
              .main-navigation .current_page_item > a,
              .main-navigation .current-menu-item > a,
              .main-navigation .current_page_ancestor > a,
              .main-navigation .current-menu-ancestor > a,
              .footer-navigation .current_page_item > a,
              .footer-navigation .current-menu-item > a,
              .footer-navigation .current_page_ancestor > a,
              .footer-navigation .current-menu-ancestor > a
              {
                color: '.$color.';
              }
            }
            
            .elementor-widget-button .elementor-widget-container .elementor-button-wrapper .elementor-button,
            .elementor-widget-button.elementor-button-info .elementor-widget-container .elementor-button-wrapper .elementor-button:hover
            {
                background-color: '.$color.';
            }
            input[type="text"]:focus,
            input[type="email"]:focus,
            input[type="url"]:focus,
            input[type="password"]:focus,
            input[type="search"]:focus,
            input[type="number"]:focus,
            input[type="tel"]:focus,
            input[type="range"]:focus,
            input[type="date"]:focus,
            input[type="month"]:focus,
            input[type="week"]:focus,
            input[type="time"]:focus,
            input[type="datetime"]:focus,
            input[type="datetime-local"]:focus,
            input[type="color"]:focus,
            textarea:focus,
            select:focus
            {
                border-color: '.$color.';
            }
            .mphb-check-in-date-wrapper input[type="text"]:active, 
            .mphb-check-in-date-wrapper input[type="text"]:focus, 
            .mphb-check-out-date-wrapper input[type="text"]:active, 
            .mphb-check-out-date-wrapper input[type="text"]:focus, 
            .mphb_sc_search-check-in-date input[type="text"]:active, 
            .mphb_sc_search-check-in-date input[type="text"]:focus, 
            .mphb_sc_search-check-out-date input[type="text"]:active, 
            .mphb_sc_search-check-out-date input[type="text"]:focus, 
            .mphb_widget_search-check-in-date input[type="text"]:active, 
            .mphb_widget_search-check-in-date input[type="text"]:focus, 
            .mphb_widget_search-check-out-date input[type="text"]:active, 
            .mphb_widget_search-check-out-date input[type="text"]:focus
            {
                background-image: url("'.get_template_directory_uri().'/img/calendar_passive_icon.svg");
            }
            select:focus {
                background-image: url("'.get_template_directory_uri().'/img/arrow_down_passive.svg");
            }
        ';


    }

    $color = get_theme_mod('alpenhouse_secondary_color', ALPENHOUSE_SECONDARY_COLOR);
    if( $color !== ALPENHOUSE_SECONDARY_COLOR ){

        $css .= '
            input[type="button"]:hover,
            input[type="reset"]:hover,
            input[type="submit"]:hover,
            button[type="submit"]:hover,
            .button:hover,
            .menu-highlighted-item a:hover,
            .more-link:hover,
            .single-post-tags .tags-links a:hover,
            .mphb-calendar .datepick-month table tbody tr td .datepick-highlight, 
            .datepick-popup .datepick-month table tbody tr td .datepick-highlight,
            .mphb-calendar .datepick-month table tbody tr td .datepick-today.datepick-highlight, 
            .datepick-popup .datepick-month table tbody tr td .datepick-today.datepick-highlight,
            .mphb-calendar .datepick-month table tbody tr td .mphb-available-date.datepick-highlight, 
            .datepick-popup .datepick-month table tbody tr td .mphb-available-date.datepick-highlight,
            .datepick-popup .datepick-month table tbody tr td a.datepick-today               
            {
                background: '.$color.';
            }            
            h1, h2, h3, h4, h5, h6,
            blockquote cite,
            blockquote small,
            th,
            input[type="button"].secondary,
            input[type="reset"].secondary,
            input[type="submit"].secondary,
            button[type="submit"].secondary,
            .button.secondary,
            .menu-highlighted-item a.secondary,
            .more-link.secondary,
            .site-header .button.secondary,
            label,
            .menu-toggle,
            .menu-toggle:hover,
            .pagination .page-numbers,
            .pagination a.prev,
            .pagination a.next,
            .post-navigation .post-title,
            .searchform .search-controls button,
            .widget_calendar .calendar_wrap table thead th,
            .widget .alpenhouse-social-menu ul li a:before,
            .blog-posts-widget-wrapper .post .entry-header h2 a,
            .pricing-item .pricing-title,
            .pricing-item .pricing,
            .site .site-wrapper .site-header .header-menu .site-branding .site-title,
            .hentry:not(.page) .entry-header h2 a,
            .entry-header .entry-title a,
            .entry-author .author .name,
            .related-posts .related-post .entry-title,
            body.archive .entry-header .entry-title a,
            .theme-social-menu a,
            .mphb-room-types .mphb-room-type .mphb-room-type-title a, 
            .mphb_sc_search_results-wrapper .mphb-room-type .mphb-room-type-title a,
            .mphb-room-types .mphb-room-type .mphb-view-details-button-wrapper a, 
            .mphb_sc_search_results-wrapper .mphb-room-type .mphb-view-details-button-wrapper a,
            .mphb-loop-room-type-attributes li:before, 
            .mphb-single-room-type-attributes li:before, 
            .mphb-widget-room-type-attributes li:before,
            .mphb-calendar .datepick-month th, 
            .mphb-calendar .datepick-month th a, 
            .datepick-popup .datepick-month th, 
            .datepick-popup .datepick-month th a,
            .mphb_sc_services-wrapper .mphb-service-title a,
            .widget.widget_mphb_rooms_widget .mphb-widget-room-type-title a,
            .site-header a,
            .mphb_sc_account .mphb-account-menu a   
            {
                color: '.$color.';
            }                
            .widget-area .tagcloud a:hover,
            .mphb-calendar .datepick-nav, .datepick-popup .datepick-nav        
            {
                background-color: '.$color.';
            }
            @media screen and (min-width: 991px) {
                .main-navigation ul ul,
                .footer-navigation ul ul{
                    background-color: '.$color.';
                }
            }
            @media screen and (max-width: 991px) {
              .main-navigation ul ul,
              .footer-navigation ul ul,
              .main-navigation li a,
              .footer-navigation li a,
              .header-menu.menu-opened .menu-toggle 
              {
                color: '.$color.';
              }
            }
            .google-maps-builder-wrap .gmb-infobubble-container {
              background-color: '.$color.' !important;
            }
            .elementor-widget-button .elementor-widget-container .elementor-button-wrapper .elementor-button:hover {
              background-color: '.$color.';
            }
            .elementor-widget-button.elementor-button-info .elementor-widget-container .elementor-button-wrapper .elementor-button {
              color: '.$color.';
            }
            .google-maps-builder-wrap .gmb-infobubble-container ~ div div ~ div {
              border-color: '.$color.' transparent transparent !important;
            }
            @media (min-width: 768px) {
              .entry-content .mphb-single-room-type-attributes .mphb-attribute-title {
                color: '.$color.';
              }
            }
        ';

    }

    $color = get_theme_mod('alpenhouse_third_color', ALPENHOUSE_THIRD_COLOR);
    if( $color !== ALPENHOUSE_THIRD_COLOR ){

        $css .= '
                    
            input[type="button"].secondary,
            input[type="reset"].secondary,
            input[type="submit"].secondary,
            button[type="submit"].secondary,
            .button.secondary,
            .menu-highlighted-item a.secondary,
            .more-link.secondary,
            .widget_calendar .calendar_wrap table thead,
            .testimonials-carousel .slick-dots li button,
            .single-post-tags .tags-links a,
            .portfolio-menu ul li a,
            .mphb-room-types .mphb-room-type .mphb-view-details-button-wrapper a, 
            .mphb_sc_search_results-wrapper .mphb-room-type .mphb-view-details-button-wrapper a,
            .mphb-calendar .datepick-month th, 
            .mphb-calendar .datepick-month th a, 
            .datepick-popup .datepick-month th, 
            .datepick-popup .datepick-month th a,
            .mphb-calendar .datepick-month table tbody tr td .datepick-today, 
            .datepick-popup .datepick-month table tbody tr td .datepick-today,
            .datepick-popup .datepick-month table tbody tr td .mphb-available-date          
            {
                background: '.$color.';
            }
            .elementor-widget-image-carousel .elementor-slick-slider .slick-dots li button:before,
            #main-slider-dots .slick-dots li button:before, #main-slider-fixed-dots .slick-dots li button:before,
            .mphb-calendar .datepick-nav .datepick-cmd, .datepick-popup .datepick-nav .datepick-cmd            
            {
                color: '.$color.';
            }     
             @media screen and (min-width: 991px) {
                .main-navigation ul ul, .footer-navigation ul ul{
                    color: '.$color.';
                }
            }
            
            .widget-area .tagcloud a
            {
                background-color: '.$color.';
            }          
            .elementor-widget-button.elementor-button-info .elementor-widget-container .elementor-button-wrapper .elementor-button {
                background-color: '.$color.';
            }  
            
        ';

    }

    $color = get_theme_mod('alpenhouse_subpages_header_overlay', ALPENHOUSE_SUBPAGES_HEADER_OVERLAY);
    $opacity = get_theme_mod('alpenhouse_subpages_header_overlay_opacity', ALPENHOUSE_SUBPAGES_HEADER_OVERLAY_OPACITY);
    if( $color !== ALPENHOUSE_SUBPAGES_HEADER_OVERLAY || $opacity !== ALPENHOUSE_SUBPAGES_HEADER_OVERLAY_OPACITY){

        $opacity_percentage = $opacity == 100 ? 1 : $opacity/100;
        $css .= '
            .site .site-wrapper .site-header .site-header-wrapper .header-overlay{
                background-color: '.$color.';
                opacity: '.$opacity_percentage.';    
            }
        ';

    }

    $color = get_theme_mod('alpenhouse_post_overlay', ALPENHOUSE_THUMBNAIL_OVERLAY);
    $opacity = get_theme_mod('alpenhouse_post_overlay_opacity', ALPENHOUSE_THUMBNAIL_OVERLAY_OPACITY);
    if( $color !== ALPENHOUSE_THUMBNAIL_OVERLAY || $opacity !== ALPENHOUSE_THUMBNAIL_OVERLAY_OPACITY){

        $opacity_percentage = $opacity == 100 ? 1 : $opacity/100;
        $rgb_color = hex2rgb($color);
        $css .= '
            .has-post-thumbnail .post-thumbnail .thumbnail-overlay{
                background: rgba('.$rgb_color.', '.$opacity_percentage.');
            }
        ';

    }


    if($css == ''){
        return false;
    }else{
        return $css;
    }

}

function alpenhouse_customizer_css(){

    if(alpenhouse_generate_customizer_css() != false){
        wp_add_inline_style('alpenhouse', alpenhouse_generate_customizer_css());
    }

}
add_action('wp_enqueue_scripts', 'alpenhouse_customizer_css');


function alpenhouse_generate_woocommerce_customizer_css(){

    $css = '';

    $color = get_theme_mod('alpenhouse_accent_color', ALPENHOUSE_ACCENT_COLOR);
    if( $color !== ALPENHOUSE_ACCENT_COLOR ){

        $css .= '
            ul.products li.product-category a:hover .woocommerce-loop-category__title,
            ul.products li.product-category a:hover .woocommerce-loop-category__title mark,
            ul.products li.product .product-wrapper .product-title-wrapper .woocommerce-loop-product__title,
            ul.products li.product .product-wrapper .star-rating span:before,
            ul.products.list li.product .product-wrapper .product-title-wrapper .woocommerce-loop-product__title:hover,
            .single-product div.product .woocommerce-product-gallery .woocommerce-product-gallery__trigger:hover,
            .single-product .woocommerce-tabs ul.tabs li a:hover,
            .single-product .woocommerce-tabs .panel .comment-respond .comment-form-rating p.stars.selected a:not(.active):before,
            .single-product .woocommerce-tabs .panel .comment-respond .comment-form-rating p.stars.selected a.active:before,
            .single-product .woocommerce-tabs .panel .comment-respond .comment-form-rating p.stars:hover a:before,
            .woocommerce-cart-form table td.product-remove a:hover,
            .star-rating span:before,
            p.stars:hover a:before,
            p.stars.selected a.active:before,
            p.stars.selected a:not(.active):before,
            .widget_product_search form button:hover, .widget_product_search form button:focus,
            .widget_shopping_cart ul li a.remove:hover,
            .widget_shopping_cart ul li a:not(.remove),
            .widget_shopping_cart ul li a:not(.remove):hover,
            .woocommerce-MyAccount-navigation ul li a:hover   
            {
                color: '.$color.';
            }
            .woocommerce-cart-form table td.actions .coupon .button:hover,
            nav.woocommerce-pagination a.page-numbers:hover,
            nav.woocommerce-pagination a.prev:hover,
            nav.woocommerce-pagination a.next:hover,
            .widget_price_filter .ui-slider .ui-slider-handle,
            .widget_price_filter .ui-slider .ui-slider-range,
            .widget_shopping_cart .buttons a:not(.checkout):hover,
            .widget_price_filter .ui-slider .ui-slider-range,
            .widget_price_filter .ui-slider .ui-slider-handle 
            {
                background: '.$color.';
            }
            .select2-dropdown ul li.select2-results__option--highlighted,
            .select2-dropdown ul li[data-selected="true"],
            .select2-dropdown ul li[data-selected="true"]:hover,
            .widget_price_filter .price_slider_amount button[type="submit"]:hover
            {
                background: '.$color.' !important;
            }
            @media screen and (min-width: 48em) {
                .site-header-cart .widget_shopping_cart .widget_shopping_cart_content ul li a:not(.remove):hover
                {
                    color: '.$color.';
                }
            }
            
        ';

    }

    $color = get_theme_mod('alpenhouse_secondary_color', ALPENHOUSE_SECONDARY_COLOR);
    if( $color !== ALPENHOUSE_SECONDARY_COLOR ){

        $css .= '
            ul.products.list li.product .product-wrapper .product-title-wrapper .woocommerce-loop-product__title,
            .single-product div.product .woocommerce-product-gallery .woocommerce-product-gallery__trigger,
            .single-product .woocommerce-tabs ul.tabs li a,
            .woocommerce-cart-form table td.actions .coupon .button,
            .woocommerce-cart-form table td.product-remove a,
            nav.woocommerce-pagination a.page-numbers, nav.woocommerce-pagination span.page-numbers,
            nav.woocommerce-pagination a.prev,
            nav.woocommerce-pagination a.next,
            .widget_product_search form button,
            .widget_shopping_cart .buttons a:not(.checkout),
            .woocommerce-MyAccount-navigation ul li a
            {
                color: '.$color.';
            }
            .onsale,
            .widget_price_filter .ui-slider .ui-slider-handle:hover, .widget_price_filter .ui-slider .ui-slider-handle.ui-state-active              
            {
                background: '.$color.';
            }
            .widget_price_filter .price_slider_amount button[type="submit"]
            {
                color: '.$color.' !important;
            }
            
        ';

    }

    $color = get_theme_mod('alpenhouse_third_color', ALPENHOUSE_THIRD_COLOR);
    if( $color !== ALPENHOUSE_THIRD_COLOR ){

        $css .= '
            .woocommerce-cart-form table td.actions .coupon .button,
            .widget_shopping_cart .buttons a:not(.checkout)
            {
                background: '.$color.';       
            }
            .widget_price_filter .price_slider_amount button[type="submit"]
            {
                background: '.$color.' !important;
            }
        ';

    }


    if($css == ''){
        return false;
    }else{
        return $css;
    }

}
