/**
 * File customizer.js.
 *
 * Theme Customizer enhancements for a better user experience.
 *
 * Contains handlers to make Theme Customizer preview reload changes asynchronously.
 */

( function( $ ) {

    "use strict";

    function append_css_to_head(id, style){
        var oldstyle = $('head').find('#'+id);
        if(oldstyle.length){
            oldstyle.remove();
            $('head').append(style);
        }else{
            $('head').append(style);
        }
    }

	// Site title and description.
	wp.customize( 'blogname', function( value ) {
		value.bind( function( to ) {
			$( '.site-title a' ).text( to );
		} );
	} );
	wp.customize( 'blogdescription', function( value ) {
		value.bind( function( to ) {
			$( '.site-description' ).text( to );
		} );
	} );

	// Header text color.
	wp.customize( 'header_textcolor', function( value ) {
		value.bind( function( to ) {
			if ( 'blank' === to ) {
				$( '.site-title, .site-description' ).css( {
					'clip': 'rect(1px, 1px, 1px, 1px)',
					'position': 'absolute'
				} );
			} else {
				$( '.site-title, .site-description' ).css( {
					'clip': 'auto',
					'position': 'relative'
				} );
				$( '.site-title a, .site-description' ).css( {
					'color': to
				} );
			}
		} );
	} );

    //Footer text
    wp.customize('alpenhouse_footer_text', function (value) {
        value.bind(function (to) {
            $('.site-info').html(to);
        })
    });

	// Theme colors
	wp.customize('alpenhouse_accent_color', function (value) {
		value.bind(function (to) {
            var id = 'customize-preview-accent-color',
                css = '<style id="'+id+'">' +
						'input[type="button"],'+
						'input[type="reset"],'+
						'input[type="submit"],'+
						'button[type="submit"],'+
						'.button,'+
						'.menu-highlighted-item a,'+
						'.more-link,'+
						'input[type="button"].secondary:hover,'+
						'input[type="reset"].secondary:hover,'+
						'input[type="submit"].secondary:hover,'+
						'button[type="submit"].secondary:hover,'+
						'.button.secondary:hover,'+
						'.menu-highlighted-item a.secondary:hover,'+
						'.more-link.secondary:hover,'+
						'.pagination a.page-numbers:hover,'+
						'.pagination a.prev:hover,'+
						'.pagination a.next:hover,'+
						'.testimonials-carousel .slick-dots li:hover button,'+
						'.testimonials-carousel .slick-dots li.slick-active button,'+
						'.elementor-widget-image-carousel .elementor-slick-slider .slick-arrow:hover,'+
						'#tribe-bar-form .tribe-bar-submit input[type="submit"],'+
						'.site .site-wrapper .site-header .site-header-wrapper .main-slider-content .slick-prev:hover,'+
						'.site .site-wrapper .site-header .site-header-wrapper .main-slider-content .slick-next:hover,'+
						'.portfolio-menu ul li.current-cat a,'+
						'.mphb-room-types .mphb-room-type .mphb-view-details-button-wrapper a:hover,'+
						'.mphb_sc_search_results-wrapper .mphb-room-type .mphb-view-details-button-wrapper a:hover,'+
						'.mphb-flexslider ul.flex-direction-nav .flex-prev:hover,'+
						'.mphb-flexslider ul.flex-direction-nav .flex-next:hover,'+
						'.mphb-calendar .datepick-nav .datepick-cmd:hover,'+
						'.datepick-popup .datepick-nav .datepick-cmd:hover,'+
						'.datepick-popup .datepick-month table tbody tr td .datepick-highlight,'+
						'.datepick-popup .datepick-month table tbody tr td .mphb-check-in-date'+
						'{'+
							'background: '+to+';'+
						'}'+
						'a, a:hover, a:focus, a:active,'+
						'.footer-navigation ul li a,'+
						'.footer-navigation ul li a:hover,'+
						'.footer-navigation ul li a:visited,'+
						'.searchform .search-controls button:hover,'+
						'.widget_calendar .calendar_wrap table tr td a,'+
						'.widget_calendar .calendar_wrap table tr td a:hover,'+
						'.widget_calendar .calendar_wrap table tfoot td,'+
						'.widget_calendar .calendar_wrap table caption,'+
						'.widget .alpenhouse-social-menu ul li a:hover:before,'+
						'.blog-posts-widget-wrapper .post .entry-header h2 a:hover,'+
						'.pricing-item .pricing-subtitle,'+
						'.elementor-widget-image-carousel .elementor-slick-slider .slick-dots li button:hover:before,'+
						'.elementor-widget-image-carousel .elementor-slick-slider .slick-dots li.slick-active button:before,'+
						'#tribe-events-content .tribe-events-tooltip h4, #tribe_events_filters_wrapper'+
						'.tribe_events_slider_val, .single-tribe_events a.tribe-events-gcal,'+
						'.single-tribe_events a.tribe-events-ical,'+
						'.site .site-wrapper .site-header .page-header-custom .breadcrumbs ul li a:hover,'+
						'.hentry:not(.page) .entry-header h2 a:hover,'+
						'.hentry:not(.page) .entry-header .featured-post,'+
						'.entry-meta a,'+
						'.entry-header .entry-title a:hover,'+
						'.related-posts .related-post a:hover .title,'+
						'.related-posts .related-post .entry-title:hover,'+
						'body.archive .entry-header .entry-title a:hover,'+
						'#main-slider-dots .slick-dots li button:hover:before, #main-slider-fixed-dots .slick-dots li button:hover:before,'+
						'#main-slider-dots .slick-dots li.slick-active button:before, #main-slider-fixed-dots .slick-dots li.slick-active button:before,'+
						'.testimonials-wrapper .testimonial-wrapper .rating .star,'+
						'.testimonials-carousel .testimonial-wrapper .rating .star,'+
						'.single-cptp-testimonial .cptp-testimonial .testimonial-wrapper .rating .star,'+
						'.theme-social-menu a:hover,'+
						'.mphb-room-types .mphb-room-type .mphb-room-type-title a:hover,'+
						'.mphb_sc_search_results-wrapper .mphb-room-type .mphb-room-type-title a:hover,'+
						'.mphb-calendar .datepick-ctrl a,'+
						'.datepick-popup .datepick-ctrl a,'+
						'.mphb_sc_services-wrapper .mphb-service-title a:hover,'+
						'.widget.widget_mphb_rooms_widget .mphb-widget-room-type-title a:hover,'+
        				'.main-navigation .current_page_item > a,'+
        				'.main-navigation .current-menu-item > a,'+
        				'.main-navigation .current_page_ancestor > a,'+
        				'.main-navigation .current-menu-ancestor > a,'+
        				'.footer-navigation .current_page_item > a,'+
        				'.footer-navigation .current-menu-item > a,'+
        				'.footer-navigation .current_page_ancestor > a,'+
        				'.footer-navigation .current-menu-ancestor > a'+
        				'.main-navigation li:hover > a,'+
        				'.main-navigation li.focus > a,'+
        				'.footer-navigation li:hover > a,'+
        				'.footer-navigation li.focus > a'+
            			'{'+
							'color: '+to+';'+
						'}'+
						'@media screen and (max-width: 991px) {'+
							'.main-navigation ul ul :hover > a,'+
							'.main-navigation ul ul .focus > a,'+
							'.footer-navigation ul ul :hover > a,'+
							'.footer-navigation ul ul .focus > a,'+
							'.main-navigation ul ul a:hover,'+
							'.main-navigation ul ul a.focus,'+
							'.footer-navigation ul ul a:hover,'+
							'.footer-navigation ul ul a.focus,'+
							'.main-navigation ul ul .current_page_item > a,'+
							'.main-navigation ul ul .current-menu-item > a,'+
							'.main-navigation ul ul .current_page_ancestor > a,'+
							'.main-navigation ul ul .current-menu-ancestor > a,'+
							'.footer-navigation ul ul .current_page_item > a,'+
							'.footer-navigation ul ul .current-menu-item > a,'+
							'.footer-navigation ul ul .current_page_ancestor > a,'+
							'.footer-navigation ul ul .current-menu-ancestor > a,'+
							'.main-navigation .current_page_item > a,'+
							'.main-navigation .current-menu-item > a,'+
							'.main-navigation .current_page_ancestor > a,'+
							'.main-navigation .current-menu-ancestor > a,'+
							'.footer-navigation .current_page_item > a,'+
							'.footer-navigation .current-menu-item > a,'+
							'.footer-navigation .current_page_ancestor > a,'+
							'.footer-navigation .current-menu-ancestor > a'+
							'{'+
								'color: '+to+';'+
							'}'+
						'}'+
						'.elementor-widget-button .elementor-button,'+
						'.elementor-widget-button.elementor-button-info .elementor-button:hover'+
						'{'+
							'background-color: '+to+' !important;'+
						'}'+
						'ul.products li.product-category a:hover .woocommerce-loop-category__title,'+
						'ul.products li.product-category a:hover .woocommerce-loop-category__title mark,'+
						'ul.products li.product .product-wrapper .product-title-wrapper .woocommerce-loop-product__title,'+
						'ul.products li.product .product-wrapper .star-rating span:before,'+
						'ul.products.list li.product .product-wrapper .product-title-wrapper .woocommerce-loop-product__title:hover,'+
						'.single-product div.product .woocommerce-product-gallery .woocommerce-product-gallery__trigger:hover,'+
						'.single-product .woocommerce-tabs ul.tabs li a:hover,'+
						'.single-product .woocommerce-tabs .panel .comment-respond .comment-form-rating p.stars.selected a:not(.active):before,'+
						'.single-product .woocommerce-tabs .panel .comment-respond .comment-form-rating p.stars.selected a.active:before,'+
						'.single-product .woocommerce-tabs .panel .comment-respond .comment-form-rating p.stars:hover a:before,'+
						'.woocommerce-cart-form table td.product-remove a:hover,'+
						'.star-rating span:before,'+
						'p.stars:hover a:before,'+
						'p.stars.selected a.active:before,'+
						'p.stars.selected a:not(.active):before,'+
						'.widget_product_search form button:hover, .widget_product_search form button:focus,'+
						'.widget_shopping_cart ul li a.remove:hover,'+
						'.widget_shopping_cart ul li a:not(.remove),'+
						'.widget_shopping_cart ul li a:not(.remove):hover,'+
						'.woocommerce-MyAccount-navigation ul li a:hover'+
						'{'+
							'color: '+to+';'+
						'}'+
						'.woocommerce-cart-form table td.actions .coupon .button:hover,'+
						'nav.woocommerce-pagination a.page-numbers:hover,'+
						'nav.woocommerce-pagination a.prev:hover,'+
						'nav.woocommerce-pagination a.next:hover,'+
						'.widget_price_filter .ui-slider .ui-slider-handle,'+
						'.widget_price_filter .ui-slider .ui-slider-range,'+
						'.widget_shopping_cart .buttons a:not(.checkout):hover,'+
						'.widget_price_filter .ui-slider .ui-slider-range,'+
						'.widget_price_filter .ui-slider .ui-slider-handle'+
						'{'+
							'background: '+to+';'+
						'}'+
						'.select2-dropdown ul li.select2-results__option--highlighted,'+
						'.select2-dropdown ul li[data-selected="true"],'+
						'.select2-dropdown ul li[data-selected="true"]:hover,'+
						'.widget_price_filter .price_slider_amount button[type="submit"]:hover'+
						'{'+
							'background: '+to+' !important;'+
						'}'+
						'@media screen and (min-width: 48em) {'+
							'.site-header-cart .widget_shopping_cart .widget_shopping_cart_content ul li a:not(.remove):hover'+
							'{'+
								'color: '+to+';'+
							'}'+
						'}'+
						'input[type="text"]:focus,'+
						'input[type="email"]:focus,'+
						'input[type="url"]:focus,'+
						'input[type="password"]:focus,'+
						'input[type="search"]:focus,'+
						'input[type="number"]:focus,'+
						'input[type="tel"]:focus,'+
						'input[type="range"]:focus,'+
						'input[type="date"]:focus,'+
						'input[type="month"]:focus,'+
						'input[type="week"]:focus,'+
						'input[type="time"]:focus,'+
						'input[type="datetime"]:focus,'+
						'input[type="datetime-local"]:focus,'+
						'input[type="color"]:focus,'+
						'textarea:focus,'+
						'select:focus'+
						'{'+
							'border-color: '+to+';'+
						'}'+
                    '</style>';

            append_css_to_head(id, css);
        })
    });

	wp.customize('alpenhouse_secondary_color', function (value) {
		value.bind(function (to) {
            var id = 'customize-preview-secondary-color',
                css = '<style id="'+id+'">' +
						'input[type="button"]:hover,'+
						'input[type="reset"]:hover,'+
						'input[type="submit"]:hover,'+
						'button[type="submit"]:hover,'+
						'.button:hover,'+
						'.menu-highlighted-item a:hover,'+
						'.more-link:hover,'+
						'.single-post-tags .tags-links a:hover,'+
						'.mphb-calendar .datepick-month table tbody tr td .datepick-highlight,'+
						'.datepick-popup .datepick-month table tbody tr td .datepick-highlight,'+
						'.mphb-calendar .datepick-month table tbody tr td .datepick-today.datepick-highlight,'+
						'.datepick-popup .datepick-month table tbody tr td .datepick-today.datepick-highlight,'+
						'.mphb-calendar .datepick-month table tbody tr td .mphb-available-date.datepick-highlight,'+
						'.datepick-popup .datepick-month table tbody tr td .mphb-available-date.datepick-highlight,'+
						'.datepick-popup .datepick-month table tbody tr td a.datepick-today'+
						'{'+
							'background: '+to+';'+
						'}'+
						'h1, h2, h3, h4, h5, h6,'+
						'blockquote cite,'+
						'blockquote small,'+
						'th,'+
						'input[type="button"].secondary,'+
						'input[type="reset"].secondary,'+
						'input[type="submit"].secondary,'+
						'button[type="submit"].secondary,'+
						'.button.secondary,'+
						'.menu-highlighted-item a.secondary,'+
						'.more-link.secondary,'+
						'.site-header .button.secondary,'+
						'label,'+
						'.menu-toggle,'+
						'.menu-toggle:hover,'+
						'.pagination .page-numbers,'+
						'.pagination a.prev,'+
						'.pagination a.next,'+
						'.post-navigation .post-title,'+
						'.searchform .search-controls button,'+
						'.widget_calendar .calendar_wrap table thead th,'+
						'.widget .alpenhouse-social-menu ul li a:before,'+
						'.blog-posts-widget-wrapper .post .entry-header h2 a,'+
						'.pricing-item .pricing-title,'+
						'.pricing-item .pricing,'+
						'.site .site-wrapper .site-header .header-menu .site-branding .site-title,'+
						'.hentry:not(.page) .entry-header h2 a,'+
						'.entry-header .entry-title a,'+
						'.entry-author .author .name,'+
						'.related-posts .related-post .entry-title,'+
						'body.archive .entry-header .entry-title a,'+
						'.theme-social-menu a,'+
						'.mphb-room-types .mphb-room-type .mphb-room-type-title a,'+
						'.mphb_sc_search_results-wrapper .mphb-room-type .mphb-room-type-title a,'+
						'.mphb-room-types .mphb-room-type .mphb-view-details-button-wrapper a,'+
						'.mphb_sc_search_results-wrapper .mphb-room-type .mphb-view-details-button-wrapper a,'+
						'.mphb-loop-room-type-attributes li:before,'+
						'.mphb-single-room-type-attributes li:before,'+
						'.mphb-widget-room-type-attributes li:before,'+
						'.mphb-calendar .datepick-month th,'+
						'.mphb-calendar .datepick-month th a,'+
						'.datepick-popup .datepick-month th,'+
						'.datepick-popup .datepick-month th a,'+
						'.mphb_sc_services-wrapper .mphb-service-title a,'+
						'.widget.widget_mphb_rooms_widget .mphb-widget-room-type-title a,'+
						'.site-header a'+
						'{'+
							'color: '+to+';'+
						'}'+
						'.main-navigation ul ul,'+
						'.footer-navigation ul ul,'+
						'.widget-area .tagcloud a:hover,'+
						'.mphb-calendar .datepick-nav, .datepick-popup .datepick-nav'+
						'{'+
							'background-color: '+to+';'+
						'}'+

						'@media screen and (max-width: 991px) {'+
							'.main-navigation ul ul,'+
							'.footer-navigation ul ul,'+
							'.main-navigation li a,'+
							'.footer-navigation li a,'+
							'.header-menu.menu-opened .menu-toggle'+
							'{'+
								'color: '+to+';'+
							'}'+
						'}'+
						'.elementor-widget-button .elementor-button:hover,'+
						'.google-maps-builder-wrap .gmb-infobubble-container{'+
							'background-color: '+to+' !important;'+
						'}'+
						'.elementor-widget-button.elementor-button-info .elementor-button {'+
							'color: '+to+' !important;'+
						'}'+
						'.google-maps-builder-wrap .gmb-infobubble-container ~ div div ~ div {'+
							'border-color: '+to+' transparent transparent !important;'+
						'}'+
						'@media (min-width: 768px) {'+
							'.entry-content .mphb-single-room-type-attributes .mphb-attribute-title {'+
								'color: '+to+';'+
							'}'+
						'}'+
						'ul.products.list li.product .product-wrapper .product-title-wrapper .woocommerce-loop-product__title,'+
						'.single-product div.product .woocommerce-product-gallery .woocommerce-product-gallery__trigger,'+
						'.single-product .woocommerce-tabs ul.tabs li a,'+
						'.woocommerce-cart-form table td.actions .coupon .button,'+
						'.woocommerce-cart-form table td.product-remove a,'+
						'nav.woocommerce-pagination a.page-numbers, nav.woocommerce-pagination span.page-numbers,'+
						'nav.woocommerce-pagination a.prev,'+
						'nav.woocommerce-pagination a.next,'+
						'.widget_product_search form button,'+
						'.widget_shopping_cart .buttons a:not(.checkout),'+
						'.woocommerce-MyAccount-navigation ul li a'+
						'{'+
							'color: '+to+';'+
						'}'+
						'.onsale,'+
						'.widget_price_filter .ui-slider .ui-slider-handle:hover, .widget_price_filter .ui-slider .ui-slider-handle.ui-state-active'+
						'{'+
							'background: '+to+';'+
						'}'+
						'.widget_price_filter .price_slider_amount button[type="submit"]'+
						'{'+
							'color: '+to+' !important;'+
						'}'+
                    '</style>';

            append_css_to_head(id, css);
        })
    });

	wp.customize('alpenhouse_third_color', function (value) {
		value.bind(function (to) {
            var id = 'customize-preview-third-color',
                css = '<style id="'+id+'">' +
				'input[type="button"].secondary,'+
				'input[type="reset"].secondary,'+
				'input[type="submit"].secondary,'+
				'button[type="submit"].secondary,'+
				'.button.secondary,'+
				'.menu-highlighted-item a.secondary,'+
				'.more-link.secondary,'+
				'.widget_calendar .calendar_wrap table thead,'+
				'.testimonials-carousel .slick-dots li button,'+
				'.single-post-tags .tags-links a,'+
				'.portfolio-menu ul li a,'+
				'.mphb-room-types .mphb-room-type .mphb-view-details-button-wrapper a,'+
				'.mphb_sc_search_results-wrapper .mphb-room-type .mphb-view-details-button-wrapper a,'+
				'.mphb-calendar .datepick-month th,'+
				'.mphb-calendar .datepick-month th a,'+
				'.datepick-popup .datepick-month th,'+
				'.datepick-popup .datepick-month th a,'+
				'.mphb-calendar .datepick-month table tbody tr td .datepick-today,'+
				'.datepick-popup .datepick-month table tbody tr td .datepick-today,'+
				'.datepick-popup .datepick-month table tbody tr td .mphb-available-date'+
				'{'+
					'background: '+to+';'+
				'}'+
				'.elementor-widget-image-carousel .elementor-slick-slider .slick-dots li button:before,'+
				'#main-slider-dots .slick-dots li button:before, #main-slider-fixed-dots .slick-dots li button:before,'+
				'.mphb-calendar .datepick-nav .datepick-cmd, .datepick-popup .datepick-nav .datepick-cmd,'+
				'.main-navigation ul ul, .footer-navigation ul ul'+
				'{'+
					'color: '+to+';'+
				'}'+
				'.widget-area .tagcloud a'+
				'{'+
					'background-color: '+to+';'+
				'}'+
				'.elementor-widget-button.elementor-button-info .elementor-button {'+
					'background-color: '+to+' !important;'+
				'}'+
				'.woocommerce-cart-form table td.actions .coupon .button,'+
				'.widget_shopping_cart .buttons a:not(.checkout)'+
				'{'+
					'background: '+to+';'+
				'}'+
				'.widget_price_filter .price_slider_amount button[type="submit"]'+
				'{'+
					'background: '+to+' !important;'+
				'}'+
            '</style>';

            append_css_to_head(id, css);
        })
    });

	wp.customize('alpenhouse_subpages_header_overlay', 'alpenhouse_subpages_header_overlay_opacity', function (color, opacity) {
		color.bind( function (value) {
            var id = 'customize-preview-subpages-header',
                css = '<style id="'+id+'">' +
					'.site .site-wrapper .site-header .site-header-wrapper .header-overlay{'+
						'background-color: '+value+';'+
						'opacity: '+opacity.get()/100+';'+
					'}'+
                    '</style>';

            append_css_to_head(id, css);
        });
		opacity.bind( function (value) {
            var id = 'customize-preview-subpages-header',
                css = '<style id="'+id+'">' +
                    '.site .site-wrapper .site-header .site-header-wrapper .header-overlay{'+
                    	'background-color: '+color.get()+';'+
                    	'opacity: '+value/100+';'+
                    '}'+
                    '</style>';

            append_css_to_head(id, css);
        });
    });

	wp.customize('alpenhouse_post_overlay', 'alpenhouse_post_overlay_opacity', function (color, opacity) {
		color.bind( function (value) {
            var id = 'customize-preview-post-overlay',
                css = '<style id="'+id+'">' +
					'.has-post-thumbnail .post-thumbnail .thumbnail-overlay{'+
						'background: '+convertHex(value, opacity.get())+';'+
					'}'+
                    '</style>';

            append_css_to_head(id, css);
        });
		opacity.bind( function (value) {
            var id = 'customize-preview-post-overlay',
                css = '<style id="'+id+'">' +
                    '.has-post-thumbnail .post-thumbnail .thumbnail-overlay{'+
                    'background: '+convertHex(color.get(), value)+';'+
                    '}'+
                    '</style>';

            append_css_to_head(id, css);
        });
    });

    function convertHex(hex,opacity){
        hex = hex.replace('#','');
        var r = parseInt(hex.substring(0,2), 16),
        	g = parseInt(hex.substring(2,4), 16),
        	b = parseInt(hex.substring(4,6), 16);


        return 'rgba('+r+','+g+','+b+','+opacity/100+')';
    }

} )( jQuery );
